import 'package:flutter_test/flutter_test.dart';
import 'package:twicpics_components/src/install.dart';
import 'package:twicpics_components/src/urls.dart';

void main() {

    group('isAbsolute', () {
        test( 'should return true', () {
            install( domain: 'https://demo.twic.pics' );
            expect( isAbsolute( src: 'https://demo.twic.pics/myasset.jpg', domain: 'https://demo.twic.pics' ), true );
        } );
        test( 'should return false', () {
            install( domain: 'https://demo.twic.pics' );
            expect( isAbsolute( src: 'image:myasset.jpg', domain: 'https://demo.twic.pics' ), false );
        } );
    } );

    group('createUrl', () {
        test( 'should create TwicPics api url from path src', () {
            expect (
                createUrl(
                    domain: 'https://demo.twic.pics',
                    src: 'media:cat.jpg'
                ),
                'https://demo.twic.pics/cat.jpg?twic=v1'
            );
        } );
        test( 'should create TwicPics api url from absolute src', () {
            expect (
                createUrl(
                    domain: 'https://demo.twic.pics',
                    src: 'https://demo.twic.pics/cat.jpg'
                ),
                'https://demo.twic.pics/cat.jpg?twic=v1'
            );
        } );
        test( 'should create TwicPics api url with transformation', () {
            expect(
                createUrl(
                    domain: 'https://demo.twic.pics',
                    src: 'media:cat.jpg',
                    transform: 'flip=x/cover=300x200',
                ),
                'https://demo.twic.pics/cat.jpg?twic=v1/flip=x/cover=300x200'
            );
        } );
        test( 'should create TwicPics api url with quality transformation', () {
            expect( 
                createUrl(
                    domain: 'https://demo.twic.pics',
                    src: 'media:cat.jpg',
                    quality: 90
                ),
                'https://demo.twic.pics/cat.jpg?twic=v1/quality=90.0'
            );
        } );
        test( 'should create TwicPics api url with output transformation', () {
            expect( 
                createUrl(
                    domain: 'https://demo.twic.pics',
                    src: 'media:cat.jpg',
                    output: 'maincolor'
                ), 
                'https://demo.twic.pics/cat.jpg?twic=v1/output=maincolor'
            );
        } );

        test( 'should create TwicPics api url for special asset', () {
            expect( 
                createUrl(
                    domain: 'https://demo.twic.pics',
                    src: 'placeholder:red',
                ),
                'https://demo.twic.pics/v1/placeholder:red'
            );
        } );
    } );
}
